@@ -26,7 +26,7 @@ import struct
 # External phonepatch modules
 import soundcard
 
-__version__ = "$Revision: 1.8 $"
+__version__ = "$Revision: 1.12 $"
 __author__ = "Arnau Sanchez <arnau@ehas.org>"
 __depends__ = ['Python-2.4']
 __copyright__ = """Copyright (C) 2006 Arnau Sanchez <arnau@ehas.org>.
@@ -97,7 +97,7 @@ class Radio:
 		self.soundcard = None
 		self.soundcard_device = soundcard_device
 		while 1:
-			try: self.soundcard = soundcard.Soundcard(device = soundcard_device, \
+			try: self.soundcard = self.open_soundcard(device = soundcard_device, \
 					channels = self.audio_channels, mode = "rw", library = "oss", \
 					samplerate = samplerate, sampleformat = self.sampleformat, \
 					fragmentsize = self.fragmentsize)
@@ -115,6 +115,17 @@ class Radio:
 		# Turn PTT off at start (for safety)
 		self.set_ptt(False)
 		
+	###################################
+	def open_soundcard(self, *args, **kwargs):
+		self.open_soundcard_args = args, kwargs
+		return soundcard.Soundcard(*args, **kwargs)
+
+	###################################
+	def reopen_soundcard(self):
+		self.soundcard.close()
+		args, kwargs = self.open_soundcard_args
+		self.soundcard = soundcard.Soundcard(*args, **kwargs)
+
 	###################################
 	def debug(self, args, exit = False):
 		"""Write logs to standard error if enabled"""
@@ -131,6 +142,11 @@ class Radio:
 		"""Get audio file descriptor used to interface soundcard"""
 		return self.soundcard
 
+	###################################
+	def get_fragmentsize(self):
+		"""Get audio file descriptor used to interface soundcard"""
+		return self.fragmentsize
+
 	#####################################
 	def limit_power(self, buffer, limit):
 		power = float(audioop.rms(buffer, self.sample_width)) / self.sample_max
@@ -151,7 +167,7 @@ class Radio:
 
 	#####################################
 	def decode_ctcss(self, buffer):
-		if not self.ctcss_decoder: return
+		if not self.ctcss_decoder or not self.carrier_state: return
 		self.ctcss_decoder.decode_buffer(buffer)
 
 	#####################################
